typedef struct {
	char *utf8;
} utf8to32_t;


/* Return next byte or UTF8_END if there's no more. */
static int get(utf8to32_t *ctx, int zero_is_legal)
{
	int c;

	if (*ctx->utf8 == '\0')
		return zero_is_legal ? 0 : -1;

	c = (*ctx->utf8) & 0xFF;
	ctx->utf8++;
	return c;
}


/* Return the 6-bit payload of the next continuation byte */
static int cont(utf8to32_t *ctx)
{
	int c = get(ctx, 0);

	if ((c & 0xC0) == 0x80)
		return (c & 0x3F);

	return -1;
}

long utf8_decode_first(char *utf8)
{
	utf8to32_t ctx;
	int c0, c1, c2, c3; /* first char continuation chrs */
	int r;

	ctx.utf8 = utf8;

	c0 = get(&ctx, 1);

	if ((c0 & 0x80) == 0) /* zero continuation (0 to 127) */
		return c0;

	if ((c0 & 0xE0) == 0xC0) { /* one continuation (128 to 2047) */
		c1 = cont(&ctx); if (c1 < 0) return -1;
		if (c1 >= 0) {
			r = ((c0 & 0x1F) << 6) | c1;
			if (r >= 128)
				return r;
		}
		return -1;
	}

	if ((c0 & 0xF0) == 0xE0) { /* Two continuations (2048 to 55295 and 57344 to 65535) */
		c1 = cont(&ctx); if (c1 < 0) return -1;
		c2 = cont(&ctx); if (c2 < 0) return -1;

		r = ((c0 & 0x0F) << 12) | (c1 << 6) | c2;
		if ((r >= 2048) && ((r < 55296) || (r > 57343)))
			return r;

		return -1;
	}

	if ((c0 & 0xF8) == 0xF0) { /* Three continuations (65536 to 1114111) */
		c1 = cont(&ctx); if (c1 < 0) return -1;
		c2 = cont(&ctx); if (c2 < 0) return -1;
		c3 = cont(&ctx); if (c3 < 0) return -1;
		if ((c1 < 0) || (c2 < 0))
			return -1;

		if ((c1 | c2 | c3) >= 0) {
			r = ((c0 & 0x07) << 18) | (c1 << 12) | (c2 << 6) | c3;
			if ((r >= 65536) && (r <= 1114111))
				return r;
		}
		return -1;
	}

	return -1;
}
