static mbtk_window_t *mbtk_xlib_id2win(xlib_display_t *xd, Window wid)
{
	return htip_get(&xd->id2win, wid);
}

static int mbtk_xlib_proc_stdin(xlib_display_t *xd, mbtk_event_t *dst)
{
	static char buf[256];
	int len;

	len = read(0, buf, sizeof(buf)-1);
	if (len > 0)
		buf[len] = '\0';
	else
		buf[0] = '\0';
	dst->type = MBTK_EV_RSTDIN;
	dst->win = ((mbtk_display_t *)xd)->topwin;
	dst->data.rstdin.len = len;
	dst->data.rstdin.data = buf;
	return 1;
}

static int mbtk_xlib_proc_xevent(xlib_display_t *xd, mbtk_event_t *dst)
{
	xlib_win_t *xw;

	XEvent event;

	while(XPending(xd->display)) {
		XNextEvent(xd->display, &event);
		switch(event.type) {
			case Expose:
				dst->type = MBTK_EV_WIN_EXPOSE;
				dst->win = mbtk_xlib_id2win(xd, event.xexpose.window);
				dst->data.win.x = event.xexpose.x;
				dst->data.win.y = event.xexpose.y;
				dst->data.win.w = event.xexpose.width;
				dst->data.win.h = event.xexpose.height;
				return 1;
			case ConfigureNotify:
				{
					int resized, moved;

					dst->win = mbtk_xlib_id2win(xd, event.xconfigure.window);
					xw = (xlib_win_t *)dst->win;

					resized = ((event.xconfigure.width != xw->last_w) || (event.xconfigure.height != xw->last_h));
					moved = ((event.xconfigure.x != xw->last_x) || (event.xconfigure.y != xw->last_y));

					if (moved) {
						dst->type = MBTK_EV_WIN_MOVE;
						xw->last_x = dst->data.win.x = event.xconfigure.x;
						xw->last_y = dst->data.win.y = event.xconfigure.y;
						dst->data.win.w = dst->data.win.h = 0;
						if (resized) {
							/* get this event again for resize; the second time moved will be 0 because we updated last_x and last_yt above */
							XPutBackEvent(xd->display, &event);
						}
						return 1;
					}
					if (resized) {
						dst->type = MBTK_EV_WIN_RESIZE;
						dst->data.win.x = dst->data.win.y = 0;
						xw->last_w = dst->data.win.w = event.xconfigure.width;
						xw->last_h = dst->data.win.h = event.xconfigure.height;
						return 1;
					}
					break; /* neither - skip */
				}
			case ClientMessage:
				if (event.xclient.data.l[0] == xd->xa.wmdel) {
					dst->type = MBTK_EV_WIN_CLOSE;
					dst->win = mbtk_xlib_id2win(xd, event.xclient.window);
					dst->data.win.x = dst->data.win.y = dst->data.win.w = dst->data.win.h = 0;
					return 1;
				}
				break;
			case KeyPress:
			case KeyRelease:
				dst->type = (event.type == KeyPress) ? MBTK_EV_KEY_PRESS : MBTK_EV_KEY_RELEASE;
				dst->win = mbtk_xlib_id2win(xd, event.xkey.window);
				mbtk_xlib_key_xlate(dst, event);
				return 1;
			case ButtonPress:
			case ButtonRelease:
				dst->win = mbtk_xlib_id2win(xd, event.xbutton.window);

				if (event.xbutton.button == 4) SCROLL(0, +1);
				if (event.xbutton.button == 5) SCROLL(0, -1);
				if (event.xbutton.button == 6) SCROLL(+1, 0);
				if (event.xbutton.button == 7) SCROLL(-1, 0);

				dst->type = (event.xbutton.type == ButtonPress) ? MBTK_EV_MOUSE_PRESS : MBTK_EV_MOUSE_RELEASE;
				if (dst->win->entered) {
					dst->data.mouse.x = event.xbutton.x;
					dst->data.mouse.y = event.xbutton.y;
				}
				else
					dst->data.mouse.x = dst->data.mouse.y = -1;
				dst->data.mouse.button = mbtk_xlib_mouse_button_xlate(event.xbutton.button);

				mbtk_display_set_mouse_button(dst->win->display, dst->data.mouse.button, (event.type == ButtonPress ? 1 : 0));

				return 1;
			case MotionNotify:
				dst->type = MBTK_EV_MOUSE_MOTION;
				dst->win = mbtk_xlib_id2win(xd, event.xmotion.window);
				dst->data.mouse.x = event.xmotion.x;
				dst->data.mouse.y = event.xmotion.y;
				dst->data.mouse.button = 0;
				return 1;
			case EnterNotify:
			case LeaveNotify:
				dst->type = (event.type == EnterNotify) ? MBTK_EV_WIN_ENTER : MBTK_EV_WIN_LEAVE;
				dst->win = mbtk_xlib_id2win(xd, event.xmotion.window);
				dst->data.mouse.x = dst->data.mouse.y = 0;
				dst->data.mouse.button = 0;
				return 1;
			case SelectionRequest:
				mbtk_xlib_clipboard_req(xd, &event);
				break;
			case SelectionClear:
				free(xd->clipboard);
				xd->clipboard = NULL;
				break;

		}
	}
	return 0;
}

static int mbtk_xlib_iterate(mbtk_display_t *display, mbtk_event_t *dst, double timeout, int block)
{
	xlib_display_t *xd = (xlib_display_t *)display;
	int sr;
	long usec;

	do {
		/* do not wait if we have pending events from X */
		if (XPending(xd->display))
			if (mbtk_xlib_proc_xevent(xd, dst))
				return 1;

		usec = timeout * 1000000.0;
		sr = mbtk_watch_fd_select(&xd->watch, dst, &usec, display->topwin);
		if (sr < 0) {
			perror("select() error");
			return -1;
		}
		if (sr > 0) {
			if ((dst->type == MBTK_EV_FDWATCH) && (dst->data.fdwatch.fd == 0)) {
				/* stdin got from fdwatch: need to read */
				if (mbtk_xlib_proc_stdin(xd, dst))
					return 1;
			}
			else if ((dst->type == MBTK_EV_FDWATCH) && (dst->data.fdwatch.fd == xd->fd)) {
				if (mbtk_xlib_proc_xevent(xd, dst))
					return 1;
			}
			else /* fd watch event, dst already filled in, just pass it through */
				return 1;
		}
	} while(block);

	return 0;
}
