#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>

static void mbtk_xlib_window_set_title(mbtk_window_t *win, const char *title)
{
	xlib_win_t *xw = (xlib_win_t *)win;
	xlib_display_t *xd = (xlib_display_t *)win->display;
	XTextProperty tp;

	if (XStringListToTextProperty((char **)&title, 1, &tp) != 0) {
		XSetTextProperty(xd->display, xw->window, &tp, XA_WM_NAME);
		XFree(tp.value);
		XFlush(xd->display);
	}
}

static mbtk_window_t *mbtk_xlib_window_new(mbtk_display_t *disp, mbtk_window_type_t type, const char *title, long x, long y, long w, long h)
{
	xlib_display_t *xd = (xlib_display_t *)disp;
	mbtk_window_t *win = calloc(sizeof(mbtk_window_t), 1);
	xlib_win_t *xw = (xlib_win_t *)win;
	XSetWindowAttributes attributes;

	if ((x < 0) || (y < 0))
		x = y = 0;

	if ((w < 0) || (h < 0))
		w = h = 100;

	attributes.background_pixel = XWhitePixel(xd->display, xd->screen);

	attributes.override_redirect = 1;

	xw->window = XCreateWindow(xd->display, XRootWindow(xd->display, xd->screen),
		x, y, w, h, 0, xd->depth,  InputOutput, xd->visual, CWBackPixel, &attributes);

	htip_set(&xd->id2win, xw->window, xw);

	win->display = disp;
	mbtk_xlib_window_set_title(win, title);

	XSelectInput(xd->display, xw->window, ExposureMask | KeyPressMask | KeyReleaseMask | ButtonReleaseMask | ButtonPressMask  | PointerMotionMask | StructureNotifyMask | EnterWindowMask | LeaveWindowMask) ;

	mbtk_xlib_window_set_type(xw, type);

	XMapWindow(xd->display, xw->window);
	XFlush(xd->display);

	xw->gc = XCreateGC(xd->display, xw->window, 0, NULL);
	xw->fnt = NULL;

	XSetWMProtocols(xd->display, xw->window, &xd->xa.wmdel, 1);

	switch(type & 0xFF) {
		case MBTK_WNTY_DROPDOWN_MENU:
		case MBTK_WNTY_POPUP_MENU:
		case MBTK_WNTY_COMBO:
			{ /* grab mouse clicks so click-elsewhere is detected while the menu is open */
				int res, n;
				for(n = 0; n < 100; n++) {
					res = XGrabPointer(xd->display, xw->window,  True,
						ButtonReleaseMask | ButtonPressMask,
						GrabModeAsync, GrabModeAsync, None, None, CurrentTime);
					if (res == GrabSuccess)
						break;
					{ /* portable sleep */
						fd_set rd, wr;
						struct timeval tv;
						FD_ZERO(&rd);
						FD_ZERO(&wr);
						tv.tv_sec = 0;
						tv.tv_usec = 50000;
						select(0, &rd, &wr, NULL, &tv);
					}
				}
				if (res != GrabSuccess)
					fprintf(stderr, "mbtk_xlib: failed to grab pointer\n");
			}
	}

	return win;
}


static void mbtk_xlib_subwindow_assign(mbtk_window_t *subwin, mbtk_window_t *direct_parent)
{
	xlib_display_t *xd = (xlib_display_t *)direct_parent->display;
	xlib_win_t *pxw = (xlib_win_t *)direct_parent;
	xlib_win_t *xw = (xlib_win_t *)subwin;
	int x = 0, y = 0, w = 100, h = 100;
	XSetWindowAttributes attributes;


	attributes.background_pixel = XWhitePixel(xd->display, xd->screen);

	attributes.override_redirect = 1;

	xw->window = XCreateWindow(xd->display, pxw->window,
		x, y, w, h, 0, xd->depth,  InputOutput, xd->visual, CWBackPixel, &attributes);

	htip_set(&xd->id2win, xw->window, xw);

	XSelectInput(xd->display, xw->window, ExposureMask | KeyPressMask | KeyReleaseMask | ButtonReleaseMask | ButtonPressMask  | PointerMotionMask | StructureNotifyMask | EnterWindowMask | LeaveWindowMask) ;

	XMapWindow(xd->display, xw->window);
	XFlush(xd->display);

	xw->gc = XCreateGC(xd->display, xw->window, 0, NULL);
	xw->fnt = NULL;
}

static void mbtk_xlib_subwindow_unassign(mbtk_window_t *subwin)
{
	xlib_display_t *xd = (xlib_display_t *)subwin->display;
	xlib_win_t *xw = (xlib_win_t *)subwin;

	XFreeGC(xd->display, xw->gc);
	htip_popentry(&xd->id2win, xw->window);
	XDestroyWindow(xd->display, xw->window);
	xw->window = 0;
}

static void mbtk_xlib_subwindow_geometry(mbtk_window_t *subwin, long x, long y, long w, long h)
{
	xlib_win_t *xw = (xlib_win_t *)subwin;
	xlib_display_t *xd = (xlib_display_t *)subwin->display;
	XMoveResizeWindow(xd->display, xw->window, x, y, w, h);
}


static mbtk_window_t *mbtk_xlib_subwindow_new(mbtk_display_t *disp, mbtk_widget_t *parent, long x, long y, long w, long h)
{
	mbtk_window_t *win = calloc(sizeof(mbtk_window_t), 1);
	xlib_win_t *xw = (xlib_win_t *)win;

	xw->sub = 1;

	return win;
}


static void mbtk_xlib_window_close(mbtk_window_t *win)
{
	xlib_win_t *xw = (xlib_win_t *)win;
	xlib_display_t *xd = (xlib_display_t *)win->display;
	htip_popentry(&xd->id2win, xw->window);
	XFreeGC(xd->display, xw->gc);
	XDestroyWindow(xd->display, xw->window);
}

static mbtk_window_t *mbtk_xlib_win2render_win(mbtk_window_t *win)
{
	return win; /* every subwindow is an X11 window */
}


static void mbtk_xlib_window_raise(mbtk_window_t *win)
{
	xlib_win_t *xw = (xlib_win_t *)win;
	xlib_display_t *xd = (xlib_display_t *)win->display;
	XEvent e = {0};

	XMapWindow(xd->display, xw->window);

	e.xany.type = ClientMessage;
	e.xclient.message_type = xd->xa.wmactw;
	e.xclient.format = 32;
	e.xclient.window = xw->window;
	e.xclient.data.l[0] = 1;
	e.xclient.data.l[1] = 0;
	e.xclient.data.l[2] = 0;

	XSendEvent(xd->display, RootWindow(xd->display, xd->screen), 0,
		SubstructureNotifyMask | SubstructureRedirectMask, &e);

	XRaiseWindow(xd->display, xw->window);
	XFlush(xd->display);
}

static void mbtk_xlib_window_hide(mbtk_window_t *win)
{
	xlib_win_t *xw = (xlib_win_t *)win;
	xlib_display_t *xd = (xlib_display_t *)win->display;

	XUnmapWindow(xd->display, xw->window);
	XFlush(xd->display);
}

static void mbtk_xlib_window_show(mbtk_window_t *win)
{
	xlib_win_t *xw = (xlib_win_t *)win;
	xlib_display_t *xd = (xlib_display_t *)win->display;

	XMapWindow(xd->display, xw->window);
	XFlush(xd->display);
}

static void mbtk_xlib_window_transient4(mbtk_window_t *win, mbtk_window_t *tr_for)
{
	mbtk_xlib_win_transient4_((xlib_win_t *)win, (xlib_win_t *)tr_for);
}

static void mbtk_xlib_window_set_min_size(mbtk_window_t *win, long w, long h)
{
	xlib_win_t *xw = (xlib_win_t *)win;
	xlib_display_t *xd = (xlib_display_t *)win->display;
	XSizeHints *sizehints = XAllocSizeHints();
	long userhints;

	XGetWMNormalHints(xd->display, xw->window, sizehints, &userhints);

	sizehints->min_width = w;
	sizehints->min_height = h;
	sizehints->flags |= PMinSize;

	XSetWMNormalHints(xd->display, xw->window, sizehints);
	XFree(sizehints);


/* SDL2 says we need to do this workaround if window is not resizable:
	XResizeWindow(xd->display, xw->window, win->w, win->h);
	XMoveWindow(xd->display, xw->window, win->x - border_left, win->y - border_top);
	XRaiseWindow(xd->display, xw->window);
*/

	XFlush(xd->display);
}

static void mbtk_xlib_window_set_size(mbtk_window_t *win, long w, long h)
{
	xlib_win_t *xw = (xlib_win_t *)win;
	xlib_display_t *xd = (xlib_display_t *)win->display;

	XResizeWindow(xd->display, xw->window, w, h);
	XFlush(xd->display);
}

