#ifndef LIBMBTK_KEYWORDS_H
#define LIBMBTK_KEYWORDS_H
#include <libmbtk/global_typedefs.h>

/* API: string->integer keywords; see doc/ref/keywords.html */


/* Convert a name to a keyword; allocate new keyword value if needed */
mbtk_kw_t mbtk_kw(const char *name);

/* Return a new unique keyword that has no name or string representation; useful
   when a handle is needed e.g. for a pixmap registration and the handle will
   be remembered by mbtk_kw_t but never resolved by string. */
mbtk_kw_t mbtk_kw_uniq(void);

/*** Internal ***/

/* Free all keywords */
void mbtk_kw_uninit(void);

#define mbtk_kw_cache(dst, name) \
do { \
	static mbtk_kw_t cache = mbtk_kw_invalid; \
	if (cache == mbtk_kw_invalid) \
		cache = mbtk_kw(name); \
	dst = cache; \
} while(0)

#endif
