#ifndef LIBMBTK_WIDGET_BOX_H
#define LIBMBTK_WIDGET_BOX_H

#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* Boxes are used as a layout tool, to arrange widgets and dynamically
   allocate space. */
typedef struct mbtk_box_s {
	mbtk_widget_t w;
	unsigned int framed:1;        /* draw frame when set */
	unsigned int no_background:1; /* do not draw background rectangle when set */
} mbtk_box_t;

mbtk_box_t *mbtk_box_new(mbtk_box_t *caller_alloced, int horiz);
mbtk_box_t *mbtk_hbox_new(mbtk_box_t *caller_alloced);
mbtk_box_t *mbtk_vbox_new(mbtk_box_t *caller_alloced);

/* Prepend or append a widget as a child of the parent widget. The parent
   widget must be a hbox or a vbox. Returns w on success or NULL on failure.
   Both insert and append are O(1). */
mbtk_widget_t *mbtk_box_add_widget(mbtk_box_t *parent, mbtk_widget_t *w, int prepend);

/* Insert a widget before or after an existing sibling. The parent
   widget must be a hbox or a vbox. Returns w on success or NULL on failure.
   NOTE: inserting before (insert=1) is O(n). Append is O(1). */
mbtk_widget_t *mbtk_box_add_widget_at(mbtk_widget_t *at, mbtk_widget_t *w, int ins_before);

/* Sets whether the box has a frame; unless style code has custom draw
   function for a widget, the fallback draw of a widget typically uses framed
   boxes to draw different sections of the widget. Boxes used for layout purposes
   re unrfamed ("invisible"). The frame widget (a thin frame with a label) is
   a differen widget. */
void mbtk_box_set_framed(mbtk_box_t *box, mbtk_bool_t has_frame);

/* When the ev2cb handler is installed on the, "event" callbacks are triggered
   on event sunhandled by child widgets. This function binds that callback. */
void mbtk_box_callback_event(mbtk_box_t *b, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_t *ev, void *user_data), void *user_data);


MBTK_WIDGET_INLINES(box)

#endif
