#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/callback_imp.h>
#include <libmbtk/widget/hand_ev2cb.h>
#include <libmbtk/widget/canvas.h>

static int mbtk_canvas_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	mbtk_canvas_t *canvas = (mbtk_canvas_t *)w;

	if (canvas->minsize != NULL)
		return canvas->minsize(canvas, mx, my);

	*mx = 1; *my = 1;
	return 0;
}

static void mbtk_canvas_post_assign(mbtk_widget_t *w)
{
	mbtk_canvas_t *canvas = (mbtk_canvas_t *)w;

	assert(canvas->subwin == NULL);
	canvas->subwin = mbtk_subwindow_new(w->win->display, &canvas->w, 0, 0, 0, 0);
	mbtk_subwindow_assign(canvas->subwin, w->render_win);
}

static void mbtk_canvas_pre_unassign(mbtk_widget_t *w)
{
	mbtk_canvas_t *canvas = (mbtk_canvas_t *)w;

	mbtk_subwindow_unassign(canvas->subwin);
	mbtk_subwindow_free(canvas->subwin);
	canvas->subwin = NULL;
}


static void mbtk_canvas_fallback_draw(mbtk_widget_t *w)
{
	mbtk_canvas_t *canvas = (mbtk_canvas_t *)w;

	/* ensure subwindow geometry matches widget's */
	mbtk_subwindow_geometry(canvas->subwin, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);

	if (canvas->draw != NULL) {
		mbtk_draw_start(canvas->subwin);
		canvas->draw(canvas, canvas->subwin->content_x0, canvas->subwin->content_y0, w->hvbox.w, w->hvbox.h);
		mbtk_draw_flush(canvas->subwin);
	}
}

static void mbtk_canvas_fallback_draw_overlay(mbtk_widget_t *w)
{
}



static const mbtk_widget_imp_t mbtk_canvas_imp = {
	NULL, /* mbtk_canvas_destroy */

	mbtk_canvas_fallback_minsize,
	mbtk_canvas_fallback_draw,
	mbtk_canvas_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	mbtk_canvas_post_assign,
	mbtk_canvas_pre_unassign
};

mbtk_canvas_t *mbtk_canvas_new(mbtk_canvas_t *caller_alloced)
{
	mbtk_canvas_t *canvas;
	mbtk_kw_t kw_canvas;

	mbtk_kw_cache(kw_canvas, "canvas");

	mbtk_create_wt(canvas, kw_canvas, &mbtk_canvas_imp, caller_alloced);

	canvas->w.relative_ev_coords = 1;
	mbtk_handler_ev2cb_install(&canvas->w);
	return canvas;
}

void mbtk_canvas_set_minsize_cb(mbtk_canvas_t *c, int (*minsize)(mbtk_canvas_t*, long*, long*))
{
	c->minsize = minsize;
}

void mbtk_canvas_set_draw_cb(mbtk_canvas_t *c, int (*draw)(mbtk_canvas_t *, long, long, long, long))
{
	c->draw = draw;
}


void mbtk_canvas_callback_event(mbtk_canvas_t *c, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_t *ev, void *user_data), void *user_data)
{
	mbtk_kw_t kw_event;
	mbtk_kw_cache(kw_event, "event");
	mbtk_callback_set_event(&c->w, kw_event, cb, user_data);
}
