#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <assert.h>
#include <libmbtk/draw.h>
#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/util.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/widget/label.h>

typedef struct {
	unsigned short width; /* in pixels */
	unsigned short next;  /* offset from the current (mbtk_label_line_t *) to the next (mbtk_label_line_t *) or 0 for the last */
	unsigned short slen;  /* string length cache */
	void *cookie;         /* backend draw */
	char s[1];            /* longer, \0 terminated */
} mbtk_label_line_t;

static void mbtk_label_free_str(mbtk_label_t *label)
{
	mbtk_label_line_t *l, *next;

	for(l = label->str; l != NULL; l = next) {
		if (l->next != 0)
			next = (mbtk_label_line_t *)((char *)l + l->next);
		else
			next = NULL;

		mbtk_draw_free_text(label->w.win, &l->cookie);
	}
	free(label->str);
	label->str = NULL;
	label->widths_valid = 0;
	label->numlines = 0;
}



static void mbtk_label_imp_destroy(mbtk_widget_t *w)
{
	mbtk_label_t *label = (mbtk_label_t *)w;

	mbtk_label_free_str(label);
}


static int mbtk_label_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	mbtk_label_minsize((mbtk_label_t *)w, mx, my);
	return 0;
}

static void mbtk_label_fallback_draw(mbtk_widget_t *w)
{
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_label_t *lab = (mbtk_label_t *)w;

	if (lab->draw_bg)
		style->draw_bg_rect(win, sp, w->flags, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);
	style->set_fg_color(win, sp, w->flags);
	mbtk_label_draw((mbtk_label_t *)w);
}

static void mbtk_label_fallback_draw_overlay(mbtk_widget_t *w)
{

}


static void mbtk_label_pre_unassign(mbtk_widget_t *w)
{
	mbtk_label_t *label = (mbtk_label_t *)w;
	mbtk_label_line_t *l, *next;

	/* free all text caches in this window */
	for(l = label->str; l != NULL; l = next) {
		if (l->next != 0)
			next = (mbtk_label_line_t *)((char *)l + l->next);
		else
			next = NULL;

		mbtk_draw_free_text(label->w.win, &l->cookie);
		l->cookie = NULL;
	}
}


static const mbtk_widget_imp_t mbtk_label_imp = {
	mbtk_label_imp_destroy,

	mbtk_label_fallback_minsize,
	mbtk_label_fallback_draw,
	mbtk_label_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	NULL, /* post_assign */
	mbtk_label_pre_unassign
};


mbtk_label_t *mbtk_label_new(mbtk_label_t *caller_alloced, const char *str)
{
	mbtk_label_t *label;
	mbtk_kw_t kw_label;

	mbtk_kw_cache(kw_label, "label");

	mbtk_create_wt(label, kw_label, &mbtk_label_imp, caller_alloced);
	label->fid = mbtk_invalid_font_id;
	mbtk_label_set_str(label, str);

	return label;
}


/* Recalculate label->lines */
static int mbtk_label_lines_recalc(mbtk_label_t *label)
{
	mbtk_label_line_t *l, *next;
	mbtk_window_t *win = label->w.win;

	if (win == NULL)
		return -1;

	for(l = label->str; l != NULL; l = next) {
		int w, asc, dsc;
		if (l->next != 0)
			next = (mbtk_label_line_t *)((char *)l + l->next);
		else
			next = NULL;
		mbtk_draw_query_text(win, &w, &asc, &dsc, l->s, strlen(l->s));
		label->font_asc = asc;
		label->font_dsc = dsc;
		l->width = w;
	}

	label->widths_valid = 1;
	return 0;
}

void mbtk_label_set_str(mbtk_label_t *label, const char *str)
{
	const char *i;
	char *o;
	long lines, totlen, slen;
	mbtk_label_line_t *curr;

	mbtk_label_free_str(label);
	label->len = strlen(str);

	for(i = str, lines = 1; *i != '\0'; i++)
		if (*i == '\n')
			lines++;

	totlen = label->len + lines * (sizeof(mbtk_label_line_t) + sizeof(void *));
	label->str = malloc(totlen);
	label->numlines = lines;

	/* pack each line into an aligned mbtk_label_line_t */
	slen = 0;
	for(i = str, curr = label->str, o = curr->s; *i != '\0'; i++) {
		assert((o - (char *)label->str) < totlen);
		if (*i == '\n') {
			long at;
			*o = '\0';
			o++;

			curr->slen = slen;
			curr->cookie = NULL;
			slen = 0;

			/* align to sizeof(void *) so that the next line struct start at the right place */
			at = o - (char *)label->str;
			at |= (sizeof(void *)-1);
			o = ((char *)label->str) + at + 1;
			o += offsetof(mbtk_label_line_t, s);

			curr->next = o - (char *)curr;
			curr = (mbtk_label_line_t *)o;
			o = curr->s;
		}
		else {
			*o = *i;
			o++;
			slen++;
		}
	}

	curr->slen = slen;
	curr->cookie = NULL;
	curr->next = 0;


	*o = '\0';
}

char *mbtk_label_get_str(mbtk_label_t *label)
{
	char *o, *i, *res = malloc(label->len+1);
	mbtk_label_line_t *l, *next;

	o = res;

	for(l = label->str; l != NULL; l = next) {
		if (l->next != 0)
			next = (mbtk_label_line_t *)((char *)l + l->next);
		else
			next = NULL;

		for(i = l->s; ; i++) {
			if (*i == '\0') {
				*o = '\n';
				o++;
				break;
			}
			*o = *i;
			o++;
		}
	}
	
	*o = '\0';
	return res;
}

static int rot90_norm(int rot90)
{
	rot90 = rot90 % 4;
	if (rot90 < 0)
		rot90 += 4;
	return rot90;
}

void mbtk_label_draw(mbtk_label_t *label)
{
	mbtk_label_line_t *l, *next;
	mbtk_window_t *win = label->w.win;
	long x1 = label->w.hvbox.x, y1 = label->w.hvbox.y;
	long x2 = label->w.hvbox.x + label->w.hvbox.w, y2 = label->w.hvbox.y + label->w.hvbox.h;
	long x0, y0, extra_h;
	int r90, a, lstep, ldx, ldy; /* line stepping */

	if (label->fid.l == -1)
		mbtk_draw_set_font(win, win->display->default_fid);
	else
		mbtk_draw_set_font(win, label->fid);

	if (!label->widths_valid) {
		if (mbtk_label_lines_recalc(label) != 0) {
			mbtk_error("mbtk_label_draw(): invalid font\n");
			return;
		}
	}

	lstep = label->font_asc + label->font_dsc;
	a = label->font_asc;

	/* calculate vertical alignment */
	if (label->valign >= 0) {
		extra_h = label->w.hvbox.h - lstep * label->numlines;
		if (label->valign == 0)
			a += (extra_h - label->font_dsc)/2;
		else
			a += extra_h - label->font_dsc;
	}

	r90 = rot90_norm(label->rot90);
	switch(r90) {
		case 0: x0 = x1; y0 = y1+a; ldx = 0; ldy = lstep; break;
		case 1: x0 = x1+a; y0 = y2; ldx = lstep; ldy = 0; break;
		case 2: x0 = x2; y0 = y2-a; ldx = 0; ldy = -lstep; break;
		case 3: x0 = x2-a; y0 = y1; ldx = -lstep; ldy = 0; break;
	}


	/* printf("label render:\n"); */
	for(l = label->str; l != NULL; l = next) {
		long x, y;
		if (l->next != 0)
			next = (mbtk_label_line_t *)((char *)l + l->next);
		else
			next = NULL;

		x = x0; y = y0;
		if (label->halign >= 0) {
			long extra_w = label->w.hvbox.w - l->width;
			int cent = (label->halign == 0);
			switch(r90) {
				case 0: if (cent) x += extra_w/2; else x += extra_w; break;
				case 1: if (cent) y -= extra_w/2; else y -= extra_w; break;
				case 2: if (cent) x -= extra_w/2; else x -= extra_w; break;
				case 3: if (cent) y += extra_w/2; else y += extra_w; break;
			}
		}

		mbtk_draw_text(win, x, y, l->s, l->slen, &l->cookie);
		/* printf(" {%p} %d '%s' %d;%d ck=%p next: %d\n", l, l->slen, l->s, x0, y0, l->cookie, l->next); */

		x0 += ldx; y0 += ldy;
	}
}

void mbtk_label_minsize(mbtk_label_t *label, long *mx, long *my)
{
	mbtk_window_t *win = label->w.win;
	mbtk_label_line_t *l, *next;
	int lstep, maxw = 0;

	if (label->fid.l == -1)
		mbtk_draw_set_font(win, win->display->default_fid);
	else
		mbtk_draw_set_font(win, label->fid);

	if (!label->widths_valid) {
		if (mbtk_label_lines_recalc(label) != 0) {
			mbtk_error("mbtk_label_draw(): invalid font\n");
			return;
		}
	}

	lstep = label->font_asc + label->font_dsc;

	for(l = label->str; l != NULL; l = next) {
		if (l->next != 0)
			next = (mbtk_label_line_t *)((char *)l + l->next);
		else
			next = NULL;
		if (l->width > maxw)
			maxw = l->width;
	}


	switch(rot90_norm(label->rot90)) {
		case 0:
		case 2:
			*mx = maxw;
			*my = lstep * label->numlines;
			break;
		case 1:
		case 3:
			*my = maxw;
			*mx = lstep * label->numlines;
			break;
	}
}

