#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/widget/menu.h>
#include <libmbtk/widget/box.h>
#include <libmbtk/widget/label.h>
#include <libmbtk/widget/hand_outclk.h>
#include <libmbtk/widget/Common.h>

/*
Design: this widget is a HBOX; each child is a menu label - normally
a label object, but can be a picture or anything else.
*/

static void mbtk_menu_imp_destroy(mbtk_widget_t *w)
{
	mbtk_menu_t *menu = (mbtk_menu_t *)w;
	mbtk_handler_hovradio_uninit(&menu->radio);
}


static const mbtk_widget_imp_t mbtk_menu_imp = {
	mbtk_menu_imp_destroy,

	mbtk_comm_frame_fallback_minsize,
	mbtk_comm_frame_fallback_draw,
	mbtk_comm_frame_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	NULL, /* post_assign */
	NULL  /* pre_unassign */
};

static mbtk_event_handled_t mbtk_menu_outclk_cb(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_menu_t *menu = user_data;
	mbtk_menu_close(menu);
	return MBTK_EVENT_HANDLED;
}

mbtk_menu_t *mbtk_menu_new(mbtk_menu_t *caller_alloced)
{
	mbtk_menu_t *menu;
	mbtk_kw_t kw_menu;
	mbtk_kw_t kw_outclk;

	mbtk_kw_cache(kw_menu, "menu");

	mbtk_create_wt(menu, kw_menu, &mbtk_menu_imp, caller_alloced);
	menu->w.hvbox.type = HVBOX_VBOX;

	mbtk_handler_hovradio_init(NULL, &menu->radio); /* win == NULL is okay here: we are going to free the handler on widget destroy */

	mbtk_handler_outclk_install(&menu->w);
	mbtk_kw_cache(kw_outclk, "outclk");
	mbtk_callback_set_noarg(&menu->w, kw_outclk, mbtk_menu_outclk_cb, menu);

	return menu;
}

static mbtk_event_handled_t mbtk_menu_radio_cb(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	int idx;
	mbtk_menu_t *menu = user_data;
	hvbox_t *m;

	/* find which selector is activated, show correspoding page, hide all other pages */
	for(m = menu->w.hvbox.ch_first, idx = 0; m != NULL; m = m->next, idx++) {
		mbtk_widget_t *wm = (mbtk_widget_t *)m;
/*		if (wm == w)
			mbtk_widget_show(wpg, 0);
		else
			mbtk_widget_hide(wpg, 0);*/
	}

	return MBTK_EVENT_HANDLED;
}

static void mbtk_menu_open_(mbtk_menu_t *menu, long screen_x, long screen_y, mbtk_window_type_t wty)
{
	mbtk_window_t *win, *parwin;

	if (menu->w.win != NULL)
		return;

	parwin = menu->parent_item->win;
	win = mbtk_window_new(parwin->display, wty, "menu", screen_x, screen_y, 32, 32);
	mbtk_window_transient4(win, parwin);

	mbtk_window_set_root_widget(win, &menu->w);

	/* auto-activate the first item */
	mbtk_hand_hovradio_activated((mbtk_widget_t *)menu->w.hvbox.ch_first);
}

void mbtk_menu_open_dropdown(mbtk_menu_t *menu, long screen_x, long screen_y)
{
	mbtk_menu_open_(menu, screen_x, screen_y, MBTK_WNTY_DROPDOWN_MENU);
}

void mbtk_menu_open_popup(mbtk_menu_t *menu, long screen_x, long screen_y)
{
	mbtk_menu_open_(menu, screen_x, screen_y, MBTK_WNTY_POPUP_MENU);
}



void mbtk_menu_close(mbtk_menu_t *menu)
{
	mbtk_window_t *win = menu->w.win;

	if (win == NULL)
		return;

	mbtk_widget_unlink(&menu->w);
	mbtk_window_close(win);
}


void mbtk_menu_append(mbtk_menu_t *menu, mbtk_widget_t *label)
{
	mbtk_kw_t kw_toggled;

	mbtk_box_add_widget((mbtk_box_t *)menu, label, 0);

	label->flags.activated = 0;

	mbtk_widget_embedded((mbtk_widget_t *)label->hvbox.parent, label);

	mbtk_handler_hovradio_add_widget(&menu->radio, label);
	mbtk_kw_cache(kw_toggled, "toggled");
	mbtk_callback_set_noarg(label, kw_toggled, mbtk_menu_radio_cb, menu);
}

void mbtk_menu_append_label(mbtk_menu_t *menu, const char *menu_name)
{
	mbtk_label_t *lab = mbtk_label_new(NULL, menu_name);
	lab->draw_bg = 1;
	mbtk_menu_append(menu, &lab->w);
}

