#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/widget/menu_bar.h>
#include <libmbtk/widget/box.h>
#include <libmbtk/widget/label.h>
#include <libmbtk/widget/Common.h>

/*
Design: this widget is a HBOX; each child is a menu label - normally
a label object, but can be a picture or anything else.
*/

static void mbtk_menu_bar_imp_destroy(mbtk_widget_t *w)
{
	mbtk_menu_bar_t *menu_bar = (mbtk_menu_bar_t *)w;
	mbtk_handler_hovradio_uninit(&menu_bar->radio);
}


static const mbtk_widget_imp_t mbtk_menu_bar_imp = {
	mbtk_menu_bar_imp_destroy,

	mbtk_comm_frame_fallback_minsize,
	mbtk_comm_frame_fallback_draw,
	mbtk_comm_frame_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	NULL, /* post_assign */
	NULL  /* pre_unassign */
};

mbtk_menu_bar_t *mbtk_menu_bar_new(mbtk_menu_bar_t *caller_alloced)
{
	mbtk_menu_bar_t *menu_bar;
	mbtk_kw_t kw_menu_bar;

	mbtk_kw_cache(kw_menu_bar, "menu_bar");

	mbtk_create_wt(menu_bar, kw_menu_bar, &mbtk_menu_bar_imp, caller_alloced);
	menu_bar->w.hvbox.type = HVBOX_HBOX;
	mbtk_widget_set_align(&menu_bar->w, HVBOX_LEFT_TOP);
	mbtk_widget_set_spacing(&menu_bar->w, 16);

	mbtk_handler_hovradio_init(NULL, &menu_bar->radio); /* win == NULL is okay here: we are going to free the handler on widget destroy */

	return menu_bar;
}

static mbtk_event_handled_t mbtk_menu_bar_radio_cb(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	int idx;
	mbtk_menu_bar_t *menu_bar = user_data;
	hvbox_t *m;

	/* find which selector is activated, show correspoding page, hide all other pages */
	for(m = menu_bar->w.hvbox.ch_first, idx = 0; m != NULL; m = m->next, idx++) {
		mbtk_widget_t *wm = (mbtk_widget_t *)m;

		if (menu_bar->menus.array[idx] != NULL) {
			if (wm->flags.activated) {
				long x = w->win->x + w->hvbox.x, y = w->win->y + w->hvbox.y + w->hvbox.h;
				mbtk_menu_open_dropdown(menu_bar->menus.array[idx], x, y);
			}
			else
				mbtk_menu_close(menu_bar->menus.array[idx]);
		}
	}

	return MBTK_EVENT_HANDLED;
}


void mbtk_menu_bar_append(mbtk_menu_bar_t *menu_bar, mbtk_widget_t *label, mbtk_menu_t *menu)
{
	mbtk_kw_t kw_toggled;

	mbtk_box_add_widget((mbtk_box_t *)menu_bar, label, 0);

	label->flags.activated = 0;

	mbtk_widget_embedded((mbtk_widget_t *)label->hvbox.parent, label);

	mbtk_handler_hovradio_add_widget(&menu_bar->radio, label);
	mbtk_kw_cache(kw_toggled, "toggled");
	mbtk_callback_set_noarg(label, kw_toggled, mbtk_menu_bar_radio_cb, menu_bar);

	if (menu != NULL)
		menu->parent_item = label;
	vtp0_append(&menu_bar->menus, menu);
}

void mbtk_menu_bar_append_label(mbtk_menu_bar_t *menu_bar, const char *menu_name, mbtk_menu_t *menu)
{
	mbtk_label_t *lab = mbtk_label_new(NULL, menu_name);
	lab->draw_bg = 1;
	mbtk_menu_bar_append(menu_bar, &lab->w, menu);
}

