#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/mpm.h>
#include <libmbtk/widget/picture.h>
#include <libmbtk/widget/label.h>
#include <libmbtk/widget/Common.h>

static void mbtk_picture_imp_destroy(mbtk_widget_t *w) { }

/* Make sure pix->pxw and pic->pxh are valid (fetch data if needed) */
static void mbtk_picture_update_pxwh(mbtk_picture_t *pic)
{
	mbtk_pixmap_t *pixmap;

	if (pic->pxwh_valid)
		return;

	pixmap = NULL;
	if (pic->pixmap_kw != mbtk_kw_invalid)
		pixmap = mbtk_mpm_get_first(pic->w.win->display, pic->pixmap_kw);

	if (pixmap != NULL) {
		pic->pxw = pixmap->w;
		pic->pxh = pixmap->h;
	}
	else
		pic->pxw = pic->pxh = 0;

	pic->pxwh_valid = 1;
}

int mbtk_picture_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	mbtk_picture_t *pic = (mbtk_picture_t *)w;

	mbtk_picture_update_pxwh(pic);

	*mx = pic->pxw;
	*my = pic->pxh;

	return 0;
}

void mbtk_picture_fallback_draw(mbtk_widget_t *w)
{
	mbtk_picture_t *pic = (mbtk_picture_t *)w;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);

	mbtk_picture_update_pxwh(pic);

	if ((pic->pixmap_kw != mbtk_kw_invalid) && (pic->pxid.l == mbtk_invalid_px_id.l))
		pic->pxid = mbtk_mpm_get(win, style, sp, pic->pixmap_kw, NULL);

	if (pic->pxid.l != mbtk_invalid_px_id.l) {
		int x0 = 0, y0 = 0;

		if ((w->hvbox.w > pic->pxw) || (w->hvbox.h > pic->pxh)) {
			/* draw background if widget is larger than the pixmap or pixmap is not found */
			style->draw_bg_rect(win, sp, w->flags, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);
			x0 = (w->hvbox.w - pic->pxw) / 2;
			y0 = (w->hvbox.h - pic->pxh) / 2;
		}
		mbtk_draw_pixmap(win, w->hvbox.x + x0, w->hvbox.y + y0, pic->pxid);
	}
	else
		style->draw_bg_rect(win, sp, w->flags, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);
}


static const mbtk_widget_imp_t mbtk_picture_imp = {
	mbtk_picture_imp_destroy,

	mbtk_picture_fallback_minsize,
	mbtk_picture_fallback_draw,
	mbtk_comm_frame_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	NULL, /* post_assign */
	NULL  /* pre_unassign */
};

mbtk_picture_t *mbtk_picture_new(mbtk_picture_t *caller_alloced, mbtk_kw_t pixmap_kw)
{
	mbtk_picture_t *pic;
	mbtk_kw_t kw_picture;

	mbtk_kw_cache(kw_picture, "picture");

	mbtk_create_wt(pic, kw_picture, &mbtk_picture_imp, caller_alloced);
	pic->pixmap_kw = pixmap_kw;
	pic->pxid = mbtk_invalid_px_id;
	pic->pxwh_valid = 0;
	return pic;
}

void mbtk_picture_set_pixmap(mbtk_picture_t *pic, mbtk_kw_t pixmap_kw)
{
	pic->pixmap_kw = pixmap_kw;
	pic->pxid = mbtk_invalid_px_id;
	pic->pxwh_valid = 0;
}

