#ifndef LIBMBTK_WIDGET_RIM_H
#define LIBMBTK_WIDGET_RIM_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>
#include <libmbtk/widget/box.h>

/* Rim: draws a rectangular rim with a title on it, around a widget.
   Title is a widget too, can be a label, a picture, a button or
   even a hbox. */
typedef struct mbtk_rim_s {
	mbtk_widget_t w;
	mbtk_box_t tbar, tspc1, dummy_title, tspc2;
} mbtk_rim_t;

/* Create a new rim with no title widget. The caller shall create a widget
   (e.g. a hbox with further widgets) and use mbtk_rim_set_child() to put
   install it in the rim, then mbtk_rim_set_title() to attach title widget(s). */
mbtk_rim_t *mbtk_rim_new(mbtk_rim_t *caller_alloced);

/* Create a new rim with a label set as title. Note: always dynamically
   alloced because of the title label widget needs to be alloced. Use
   mbtk_rim_new_with_widget() for static allocation. */
mbtk_rim_t *mbtk_rim_new_with_label(const char *str);

/* Create a new rim with a widget set as title. Can be used with static
   allocation, for example:
   mbtk_rim_new_with_widget(&w1, mbtk_label_new(&w2, "foo")); */
mbtk_rim_t *mbtk_rim_new_with_widget(mbtk_rim_t *caller_alloced, mbtk_widget_t *title);


/* A rim hosts a single widget (which can be a container, e.g. hbox or vbox).
   If the rim already had a child widget, it is unlinked (removed from the
   rim) but not freed.
   Returns original child on success (may be NULL). */
mbtk_widget_t *mbtk_rim_set_child(mbtk_rim_t *parent, mbtk_widget_t *w);


/* A rim's title is a single widget (which can be a container, e.g. hbox).
   If the rim already had a title widget, it is unlinked (removed from the
   rim) but not freed.
   Returns original title on success (may be NULL). */
mbtk_widget_t *mbtk_rim_set_title_widget(mbtk_rim_t *parent, mbtk_widget_t *title);
mbtk_widget_t *mbtk_rim_set_title_label(mbtk_rim_t *parent, const char *str);

/* Return the named subwidget of rim:
    - tbar is the hbox of the titlebar
    - tspc1 is the vbox used as a placeholder in the titlebar left of the title
    - tspc2 is the vbox used as a placeholder in the titlebar right of the title
    - title is the (application supplied) title widget, typically a label
    - content is the widget within the rim */
mbtk_widget_t *mbtk_rim_subwidget(mbtk_rim_t *rim, mbtk_kw_t subname);

MBTK_WIDGET_INLINES(rim)

#endif
